<?php
/*
Plugin Name: Blip Widget
Plugin URI: http://blog.greenek.com/2010/01/04/blip-widget/
Description: Widget wyswietlajacy ostatnie wpisy uzytkownika z serwisu Blip.pl. Swietnie nadaje sie jako zamiennik minibloga.
Version: 0.4.1
Author: Greenek
Author URI: http://blog.greenek.com/
*/

// Domyślne ustawienia
$blip_widget_defaults = array(
    'username'  => 'blipinfo',
    'limit'     => 5,
    'cache_lifetime'    => 0,
    'type'      => 'html',
    'status_format' => '<li><em class="blip-date"><a href="{link}">{date[H:i, d.m]}</a></em> <span class="blip-entry">{status}</span></li>',
);

function get_recent_blips($options = array()) {
    global $blip_widget_defaults;

    // Sprawdź, czy allow_url_fopen jest włączone.
    if ( ! is_allow_url_fopen_enabled())
    {
        echo '<p>Przepraszamy, ale <em>allow_url_fopen</em> jest wyłączone na tym serwerze, w związku z czym Blip-Widget nie może działać. Skontaktuj się z administratorem albo (jeśli serwer posiada włączony cURL) skorzystaj z rozszerzenia <a href="http://wordpress.org/extend/plugins/wp-blip/">WP-Blip</a>.</p>';

        return false;
    }

    // Przypisz opcje
    $username = (isset($options['username']))
        ? $options['username']
        : $blip_widget_defaults['username'];

    $limit = (isset($options['limit']))
        ? $options['limit']
        : $blip_widget_defaults['limit'];

    $cache_lifetime = (isset($options['cache_lifetime']))
        ? $options['cache_lifetime']
        : $blip_widget_defaults['cache_lifetime'];

    $html = (isset($options['type']) AND $options['type'] == 'html')
        ? TRUE
        : FALSE;

    $status_format = (isset($options['status_format']) AND $options['status_format'] != '')
        ? $options['status_format']
        : $blip_widget_defaults['status_format'];

    $blips = '';

    // Jeśli cache jest włączony
    if ($cache_lifetime > 0)
    {
        // Sprawdź datę
        $last_cache = get_option('blip_widget_last_cache');
        $next_cache = $last_cache + ($cache_lifetime * 60);

        // Pobierz statusy z cache'u
        $blips = get_option('blip_widget_cache');
    }

    if ($blips == '' OR ($cache_lifetime > 0 AND date('U') > $next_cache))
    {
	    $blips = '';

        // Pobierz feed
        $url = 'http://'.$username.'.blip.pl/feed';

        $feed = '';
        $handle = @fopen($url, "r");
        if ($handle)
        {
            while ( ! feof($handle))
            {
                $feed .= fgets($handle, 4096);
            }
            fclose($handle);
        }

        // Jeśli nie udało się pobrać feed'a...
        if (empty($feed))
        {
            // ... pobierz go z cachu.
            $feed = get_option('blip_widget_feed');
        }

        // Na wypadek przeciążenia blip.pl zapisz feed w bazie
        update_option('blip_widget_feed', $feed);

        $xml = simplexml_load_string($feed);

        // Jeśli mimo wszystko feed jest pusty...
        if ($xml === false)
        {
            echo '<p>Wygląda na to, że Blip.pl jest w tym momencie przeciążony.</p>';
            return false;
        }

        $i = 1;

        // Parsowanie
        foreach ($xml->entry as $entry)
        {
            if ($html)
            {
                // Zamień tagi, obrazy i video na linki
                $patterns = array(
                    '/<a href="([[:alnum:]:@#%\&_=?\/\.-]+)" title="Statusy oznaczone tagiem: ([[:alnum:]]+)">/',
                    '/<div class="photo"> <a href="([[:alnum:]:@#%\&_=?\/\.-]+)"><img src="([[:alnum:]:@#%\&_=?\/\.-]+)" alt="default picture" \/><\/a><\/div>/',
                    '/<div class="video youtube">(.+)youtube.com\/v\//s',
                    '/" type="application\/x-shockwave-flash"(.+)<\/div>/s'
                );

                $replacements = array(
                    '<a href="\\1" title="Statusy oznaczone tagiem: \\2" class="blip-tag">',
                    ' <a href="\\1" class="blip-photo">[Foto]</a>',
                    ' <a href="http://www.youtube.com/watch?v=',
                    '" class="blip-video">[YouTube]</a>'
                );

                $content = preg_replace($patterns, $replacements, $entry->content);
            }
            else
            {
                $content = $entry->title;
            }

            // Data aktualizacji
            $updated = get_date_from_gmt(date('Y-m-d H:i:s', strtotime($entry->updated)));

            // Znajdź ID statusu
            preg_match('/tag:'.$username.'.blip.pl,[0-9]{4}-[0-9]{2}-[0-9]{2}:([0-9]+)/', $entry->id, $id);
            $id = $id[1];

            // Zamień znaczniki
            $patterns = array(
                '/{status}/',
                '/{date\[(.*)\]}/e',
                '/{link}/'
            );

            $replacements = array(
                $content,
                "date_i18n('$1', strtotime('$updated'))",
                'http://blip.pl/s/'.$id
            );

            $blips .= preg_replace($patterns, $replacements, $status_format);

            if (++$i > $limit)
                break;
        }

        // Jeśli cache jest włączony...
        if ($cache_lifetime > 0)
        {
            // ... zapisz do bazy.
            update_option('blip_widget_last_cache', date('U'));
            update_option('blip_widget_cache', $blips);
        }

        // Link do bliplogu użytkownika
        $blips .= '<p><a href="http://'.$username.'.blip.pl/">Zobacz mój Bliplog &raquo;</a></p>';
    }

    echo $blips;
}

/**
 * Sprawdza, czy allow_url_fopen jest włączone w konfiguracji serwera.
 */
function is_allow_url_fopen_enabled()
{
    return (ini_get('allow_url_fopen') == 1) ? true : false;
}

class Blip_Widget extends WP_Widget {

    function Blip_Widget()
    {
        $widget_ops = array('classname' => 'widget_blip', 'description' => 'Ostatnie statusy z serwisu blip.pl');
        $control_ops = array();

        parent::WP_Widget(false, $name = 'Blip.pl', $widget_ops);
    }

    function widget($args, $instance)
    {
        // Sprawdź, czy allow_url_fopen jest włączone.
        if ( ! is_allow_url_fopen_enabled())
            return false;

        extract($args);

        $title = esc_attr($instance['title']);

        $options = array(
            'username' => esc_attr($instance['username']),
            'limit' => esc_attr($instance['limit']),
            'cache_lifetime' => esc_attr($instance['cache_lifetime']),
            'type' => esc_attr($instance['type']),
            'status_format' => $instance['status_format'],
        );

        echo $before_widget;
        if ( !empty($title) ) { echo $before_title . $title . $after_title; }
            echo '<ul id="blip-widget">';
            get_recent_blips($options);
            echo '</ul>';
        echo $after_widget;
    }

    function update($new_instance, $old_instance)
    {
        update_option('blip_widget_cache', '');
        return $new_instance;
    }

    function form($instance)
    {
        global $blip_widget_defaults;

        $instance = wp_parse_args( (array) $instance, array(
            'title' => 'Blip.pl',
            'username' => '',
            'limit' => $blip_widget_defaults['limit'],
            'cache_lifetime' => $blip_widget_defaults['cache_lifetime'],
            'type' => $blip_widget_defaults['type'],
            'status_format' => ''
        ));

        // Sprawdź, czy allow_url_fopen jest włączone.
        if ( ! is_allow_url_fopen_enabled())
        {
            echo '<p>Przepraszamy, ale <em>allow_url_fopen</em> jest wyłączone na tym serwerze, w związku z czym Blip-Widget nie może działać. Skontaktuj się z administratorem albo (jeśli serwer posiada włączony cURL) skorzystaj z rozszerzenia <a href="http://wordpress.org/extend/plugins/wp-blip/">WP-Blip</a>.</p>';

            return false;
        }

        $title = esc_attr($instance['title']);
        $username = esc_attr($instance['username']);
        $limit = esc_attr($instance['limit']);
        $cache_lifetime = esc_attr($instance['cache_lifetime']);
        $type = esc_attr($instance['type']);
        $status_format = esc_attr($instance['status_format']);

        ?>
            <p><label for="<?php echo $this->get_field_id('title') ?>"><?php _e('Title:') ?> <input class="widefat" id="<?php echo $this->get_field_id('title') ?>" name="<?php echo $this->get_field_name('title') ?>" type="text" value="<?php echo $title ?>" /></label></p>

            <p><label for="<?php echo $this->get_field_id('username') ?>">Nazwa użytkownika: <input class="widefat" id="<?php echo $this->get_field_id('username') ?>" name="<?php echo $this->get_field_name('username') ?>" type="text" value="<?php echo $username ?>" /></label></p>

            <p><label for="<?php echo $this->get_field_id('limit') ?>">Limit wpisów: <select class="widefat" id="<?php echo $this->get_field_id('limit') ?>" name="<?php echo $this->get_field_name('limit') ?>" >
            <?php for($i = 1; $i <= 20; $i++): ?>
                <?php $selected = ($i == $limit) ? ' selected="selected"' : '' ?>
                <option value="<?php echo $i ?>"<?php echo $selected ?>><?php echo $i ?></option>
            <?php endfor ?>
            </select></p>

            <p><label for="<?php echo $this->get_field_id('cache_lifetime') ?>">Długość przechowywania cachu w pamięci (w min.). 0 - brak cachu: <input class="widefat" id="<?php echo $this->get_field_id('cache_lifetime') ?>" name="<?php echo $this->get_field_name('cache_lifetime') ?>" type="text" value="<?php echo $cache_lifetime ?>" /></label></p>

            <p><label for="<?php echo $this->get_field_id('type') ?>">Wyświetlaj jako: <select class="widefat" id="<?php echo $this->get_field_id('type') ?>" name="<?php echo $this->get_field_name('type') ?>" >
                <option value="html" <?php if ($type == 'html') echo 'selected="selected"' ?>>HTML</option>
                <option value="plain" <?php if ($type == 'plain') echo 'selected="selected"' ?>>Tekst</option>
            </select></p>

            <p><label for="<?php echo $this->get_field_id('status_format') ?>">Format wyświetlania statusu <em>(opcjonalnie)</em>: <input class="widefat" id="<?php echo $this->get_field_id('status_format') ?>" name="<?php echo $this->get_field_name('status_format') ?>" type="text" value="<?php echo $status_format ?>" /></label>
            <small>Znaczniki: <em>{date[format]}</em>, <em>{status}</em>, domyślnie <code><?php echo htmlspecialchars($blip_widget_defaults['status_format']) ?></code>. <a href="http://wordpress.org/extend/plugins/blip-widget/installation/">Zobacz pełną pomoc</a>.</small></p>
        <?php
    }

}

add_action('widgets_init', create_function('', 'return register_widget("Blip_Widget");'));

?>
